sbit LCD_RS at RD0_bit;
sbit LCD_EN at RD2_bit;
sbit LCD_D7 at RB7_bit;
sbit LCD_D6 at RB6_bit;
sbit LCD_D5 at RB5_bit;
sbit LCD_D4 at RB4_bit;

sbit LCD_RS_Direction at TRISD0_bit;
sbit LCD_EN_Direction at TRISD2_bit;
sbit LCD_D7_Direction at TRISB7_bit;
sbit LCD_D6_Direction at TRISB6_bit;
sbit LCD_D5_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB4_bit;

//program PICNonSPI

sbit psxCLK at RB3_bit;   //Red
sbit psxCMD at RB2_bit;   //Yellow
sbit psxDAT at RB1_bit;   //Green
sbit psxATT at RB0_bit;   //Orange
//Black - GND; Brown - VCC

unsigned char psxOut, psxIn, psxID,i,psxStatus,psxLeftB,psxRightB,psxRJoyX,psxRJoyY,psxLJoyX,psxLJoyY;

////////////////////////////////////
//       Sub Procedures           //
////////////////////////////////////

// Send and Receive data simultaneously
unsigned char psxTxRx(unsigned char byteOut, unsigned char byteIn)
{
    byteIn = 0;                        //Reset Receive byte
    for(i = 0; i<7; i++)
    {                  //Read data bits LSB to MSB
      psxCMD = byteOut & 0x01;   //Prepare first bit to send
      byteOut >>=1;
      psxCLK = 0;                    //Generate clock pulse
      delay_us(25);                //Used to regulate communication speed
      if(psxDAT == 0)
      {
         byteIn = byteIn | (0x01<<i);
         //Setbit(byteIn,i);         //Low Data line indicates set bit
      }
      psxCLK = 1;                    //End clock pulse
      Delay_us(25);                //Regulate speed
    }
    return  byteIn;
}

/////////////////////////////////////
//        Program Start            //
/////////////////////////////////////

void main()
{
  OSCCON = 0b01111110;               // 'Set INTRC to 8Mhz
  //ANSEL = 0;                         //'Set I/O to digital

  TRISA = 0;                         //'PortA all outputs
  TRISD = 0;
  //TRISC = 0;
  PORTA = 0;                         //'PortA all off
  PORTD = 0x00;
  TRISB = 0b00000010;                // 'PortB all output except psxDAT Pin (RB1)
  PORTB = 0b00001001;                // 'PortB psxCLK and psxAtt are high, all else low
  //RD1_bit = 1;
  Lcd_Init();
  RA0_bit = 1;                       // 'Flashes LED on RA0 for debugging purposes
  Delay_ms(500);
  RA0_bit = 0;
  Delay_ms(500);
  RA0_bit = 1;
  Delay_ms(200);
  RA0_bit = 0;
  Delay_ms(200);


  /////////////////////////////////////
  //           main Loop             //
  /////////////////////////////////////

  do
  {
    psxATT = 0;                        // 'psxAtt low, Activates Controller
    Delay_us(50);

    psxOut = 0x01;                     // 'Start Signal
    psxTxRx(psxOut,psxIn);

    psxOut = 0x42;                     // 'Request Status
    psxID = psxTxRx(psxOut,psxIn);
    //psxIn;                     // 'Simultaneously receive controller ID

    psxOut = 0;                        // 'Clear psxOut
    psxStatus = psxTxRx(psxOut,psxIn);             // 'Get psxStatus
    //psxIn;

    psxLeftB = psxTxRx(psxOut,psxIn);             // 'Get Left side buttons
    //psxIn;

    psxRightB = psxTxRx(psxOut,psxIn);             // 'Get Right side buttons
    //psxIn;

    psxRJoyX = psxTxRx(psxOut,psxIn);             // 'Get Right joystick X-axis byte
    //psxIn;

    psxRJoyY = psxTxRx(psxOut,psxIn);             // 'Get Right joystick Y-axis byte
    //psxIn;

    psxLJoyX = psxTxRx(psxOut,psxIn);             // 'Get Left joystick X-axis byte
    //psxIn;

    psxLJoyY = psxTxRx(psxOut,psxIn);             // 'Get Left joystick Y-axis byte
    //psxIn;

    Delay_us(50);
    psxATT = 1;                        // 'Release controller by setting psxATT high

    RA0_bit = psxRightB & 0x02;        // 'RA0 equals status of "X" Button
    RA1_bit = psxRightB & 0x04;        // 'RA1 equals status of "O" Button
    Delay_ms(10);
    Lcd_Out(1, 1, "psxID   = ");
    Lcd_Chr_CP(psxID+48);
    Delay_ms(1000);
    Lcd_Out(1, 1, "psxStat = ");
    Lcd_Chr_CP(psxStatus+48);
    Delay_ms(500);
    Lcd_Out(1, 1, "psxLB   = ");
    Lcd_Chr_CP(psxLeftB+48);
    Delay_ms(500);
    Lcd_Out(1, 1, "psxRB   = ");
    Lcd_Chr_CP(psxRightB+48);
    Delay_ms(500);
    Lcd_Out(1, 1, "psxRJoyX= ");
    Lcd_Chr_CP(psxRJoyX+48);
    Delay_ms(500);
    Lcd_Out(1, 1, "psxRJoyY= ");
    Lcd_Chr_CP(psxRJoyY+48);
    Delay_ms(500);
    Lcd_Out(1, 1, "psxLJoyX= ");
    Lcd_Chr_CP(psxLJoyX+48);
    Delay_ms(500);
    Lcd_Out(1, 1, "psxLJoyY= ");
    Lcd_Chr_CP(psxLJoyY+48);
    Delay_ms(500);
  }
  while(1);//loop until false                 'Loop Forever
  //end.

}

